@@ -1,4 +1,4 @@
-/* $Id: main.c,v 1.8 2002/01/28 16:39:30 kevinv Exp $ */
+/* $Id: main.c,v 1.9 2002/01/30 03:25:30 kevinv Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -28,13 +28,15 @@ gweather_applet_new(PanelApplet *applet, const gchar *iid, gpointer data)
 {
 	GWeatherApplet *gw_applet;
 
+	panel_applet_add_preferences(applet, "/schemas/apps/gweather/prefs", NULL);
+	
 	gw_applet = g_new0(GWeatherApplet, 1);   
 	
 	gw_applet->applet = applet;
 
     	gweather_applet_create(gw_applet);
 
-    	gweather_pref_load("test_path", gw_applet);
+    	gweather_pref_load(gw_applet);
     	gweather_info_load("test_path", gw_applet);
     
     	gweather_update(gw_applet);
